@@ -6,8 +6,10 @@ from sqlalchemy.dialects.mssql import pyodbc, pymssql, adodbapi
 from sqlalchemy.engine import url
 from sqlalchemy.testing import fixtures
 from sqlalchemy import testing
-from sqlalchemy.testing import assert_raises_message, assert_warnings
+from sqlalchemy.testing import assert_raises_message, \
+    assert_warnings, expect_warnings
 from sqlalchemy.testing.mock import Mock
+from sqlalchemy.dialects.mssql import base
 
 
 class ParseConnectTest(fixtures.TestBase):
@@ -42,7 +44,9 @@ class ParseConnectTest(fixtures.TestBase):
 
     def test_pyodbc_hostname(self):
         dialect = pyodbc.dialect()
-        u = url.make_url('mssql://username:password@hostspec/database?driver=SQL+Server')
+        u = url.make_url(
+            'mssql://username:password@hostspec/database?driver=SQL+Server'
+        )
         connection = dialect.create_connect_args(u)
         eq_([['DRIVER={SQL Server};Server=hostspec;Database=database;UI'
             'D=username;PWD=password'], {}], connection)
@@ -88,9 +92,9 @@ class ParseConnectTest(fixtures.TestBase):
         eq_(connection[1], {})
         eq_(connection[0][0]
             in ('DRIVER={SQL Server};Server=hostspec;Database=database;'
-            'UID=username;PWD=password;foo=bar;LANGUAGE=us_english',
-            'DRIVER={SQL Server};Server=hostspec;Database=database;UID='
-            'username;PWD=password;LANGUAGE=us_english;foo=bar'), True)
+                'UID=username;PWD=password;foo=bar;LANGUAGE=us_english',
+                'DRIVER={SQL Server};Server=hostspec;Database=database;UID='
+                'username;PWD=password;LANGUAGE=us_english;foo=bar'), True)
 
     def test_pyodbc_odbc_connect(self):
         dialect = pyodbc.dialect()
@@ -174,7 +178,7 @@ class ParseConnectTest(fixtures.TestBase):
         connection = dialect.create_connect_args(u)
         eq_(
             [[], {'host': 'somehost', 'password': 'tiger',
-                    'user': 'scott', 'database': 'test'}], connection
+                  'user': 'scott', 'database': 'test'}], connection
         )
 
         u = \
@@ -182,7 +186,7 @@ class ParseConnectTest(fixtures.TestBase):
         connection = dialect.create_connect_args(u)
         eq_(
             [[], {'host': 'somehost:5000', 'password': 'tiger',
-                    'user': 'scott', 'database': 'test'}], connection
+                  'user': 'scott', 'database': 'test'}], connection
         )
 
     def test_pymssql_disconnect(self):
@@ -195,11 +199,40 @@ class ParseConnectTest(fixtures.TestBase):
                 'Error 10054',
                 'Not connected to any MS SQL server',
                 'Connection is closed'
-                ]:
+        ]:
             eq_(dialect.is_disconnect(error, None, None), True)
 
         eq_(dialect.is_disconnect("not an error", None, None), False)
 
+    def test_pyodbc_disconnect(self):
+        dialect = pyodbc.dialect()
+
+        class MockDBAPIError(Exception):
+            pass
+
+        class MockProgrammingError(MockDBAPIError):
+            pass
+
+        dialect.dbapi = Mock(
+            Error=MockDBAPIError, ProgrammingError=MockProgrammingError)
+
+        for error in [
+            MockDBAPIError("[%s] some pyodbc message" % code)
+            for code in [
+                '08S01', '01002', '08003', '08007',
+                '08S02', '08001', 'HYT00', 'HY010']
+        ] + [
+            MockProgrammingError(message)
+            for message in [
+                "(some pyodbc stuff) The cursor's connection has been closed.",
+                "(some pyodbc stuff) Attempt to use a closed connection."
+            ]
+        ]:
+            eq_(dialect.is_disconnect(error, None, None), True)
+
+        eq_(dialect.is_disconnect(
+            MockProgrammingError("not an error"), None, None), False)
+
     @testing.requires.mssql_freetds
     def test_bad_freetds_warning(self):
         engine = engines.testing_engine()
@@ -239,4 +272,92 @@ class VersionDetectionTest(fixtures.TestBase):
             eq_(
                 dialect._get_server_version_info(conn),
                 (11, 0, 9216, 62)
-            )
\ No newline at end of file
+            )
+
+    def test_pyodbc_version_productversion(self):
+        dialect = pyodbc.MSDialect_pyodbc()
+
+        conn = Mock(scalar=Mock(return_value="11.0.9216.62"))
+        eq_(
+            dialect._get_server_version_info(conn),
+            (11, 0, 9216, 62)
+        )
+
+    def test_pyodbc_version_fallback(self):
+        dialect = pyodbc.MSDialect_pyodbc()
+        dialect.dbapi = Mock()
+
+        for vers, expected in [
+            ("11.0.9216.62", (11, 0, 9216, 62)),
+            ("notsqlserver.11.foo.0.9216.BAR.62", (11, 0, 9216, 62)),
+            ("Not SQL Server Version 10.5", (5, ))
+        ]:
+            conn = Mock(
+                scalar=Mock(
+                    side_effect=exc.DBAPIError("stmt", "params", None)),
+                connection=Mock(
+                    getinfo=Mock(return_value=vers)
+                )
+            )
+
+            eq_(
+                dialect._get_server_version_info(conn),
+                expected
+            )
+
+
+class IsolationLevelDetectTest(fixtures.TestBase):
+
+    def _fixture(self, view):
+        class Error(Exception):
+            pass
+
+        dialect = pyodbc.MSDialect_pyodbc()
+        dialect.dbapi = Mock(Error=Error)
+        dialect.server_version_info = base.MS_2012_VERSION
+
+        result = []
+
+        def fail_on_exec(stmt, ):
+            if view is not None and view in stmt:
+                result.append(('SERIALIZABLE', ))
+            else:
+                raise Error("that didn't work")
+
+        connection = Mock(
+            cursor=Mock(
+                return_value=Mock(
+                    execute=fail_on_exec,
+                    fetchone=lambda: result[0]
+                ),
+            )
+        )
+
+        return dialect, connection
+
+    def test_dm_pdw_nodes(self):
+        dialect, connection = self._fixture("dm_pdw_nodes_exec_sessions")
+
+        eq_(
+            dialect.get_isolation_level(connection),
+            "SERIALIZABLE"
+        )
+
+    def test_exec_sessions(self):
+        dialect, connection = self._fixture("exec_sessions")
+
+        eq_(
+            dialect.get_isolation_level(connection),
+            "SERIALIZABLE"
+        )
+
+    def test_not_supported(self):
+        dialect, connection = self._fixture(None)
+
+        with expect_warnings("Could not fetch transaction isolation level"):
+            assert_raises_message(
+                NotImplementedError,
+                "Can't fetch isolation",
+                dialect.get_isolation_level, connection
+            )
+
