@@ -7,6 +7,7 @@ from sqlalchemy import testing
 from sqlalchemy import Sequence, Table, Column, Integer, update, String,\
     func, MetaData, Enum, Index, and_, delete, select, cast, text, \
     Text, null
+from sqlalchemy import types as sqltypes
 from sqlalchemy.dialects.postgresql import ExcludeConstraint, array
 from sqlalchemy import exc, schema
 from sqlalchemy.dialects import postgresql
@@ -16,6 +17,8 @@ from sqlalchemy.sql import table, column, operators, literal_column
 from sqlalchemy.sql import util as sql_util
 from sqlalchemy.util import u, OrderedDict
 from sqlalchemy.dialects.postgresql import aggregate_order_by, insert
+from sqlalchemy.dialects.postgresql import array_agg as pg_array_agg
+from sqlalchemy.dialects.postgresql import ARRAY as PG_ARRAY
 
 
 class SequenceTest(fixtures.TestBase, AssertsCompiledSQL):
@@ -233,6 +236,26 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             'CREATE TABLE atable (id INTEGER) INHERITS '
             '( "Quote Me", "quote Me Too" )')
 
+    def test_create_table_partition_by_list(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer), Column("part_column", Integer),
+            postgresql_partition_by='LIST (part_column)')
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            'CREATE TABLE atable (id INTEGER, part_column INTEGER) '
+            'PARTITION BY LIST (part_column)')
+
+    def test_create_table_partition_by_range(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer), Column("part_column", Integer),
+            postgresql_partition_by='RANGE (part_column)')
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            'CREATE TABLE atable (id INTEGER, part_column INTEGER) '
+            'PARTITION BY RANGE (part_column)')
+
     def test_create_table_with_oids(self):
         m = MetaData()
         tbl = Table(
@@ -316,6 +339,32 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                             '(data text_pattern_ops, data2 int4_ops)',
                             dialect=postgresql.dialect())
 
+    def test_create_index_with_labeled_ops(self):
+        m = MetaData()
+        tbl = Table('testtbl', m,
+                    Column('data', String),
+                    Column('data2', Integer, key='d2'))
+
+        idx = Index('test_idx1', func.lower(tbl.c.data).label('data_lower'),
+                    postgresql_ops={'data_lower': 'text_pattern_ops'})
+
+        idx2 = Index(
+            'test_idx2',
+            (func.xyz(tbl.c.data) + tbl.c.d2).label('bar'),
+            tbl.c.d2.label('foo'),
+            postgresql_ops={'bar': 'text_pattern_ops',
+                            'foo': 'int4_ops'})
+
+        self.assert_compile(schema.CreateIndex(idx),
+                            'CREATE INDEX test_idx1 ON testtbl '
+                            '(lower(data) text_pattern_ops)',
+                            dialect=postgresql.dialect())
+        self.assert_compile(schema.CreateIndex(idx2),
+                            'CREATE INDEX test_idx2 ON testtbl '
+                            '((xyz(data) + data2) text_pattern_ops, '
+                            'data2 int4_ops)',
+                            dialect=postgresql.dialect())
+
     def test_create_index_with_text_or_composite(self):
         m = MetaData()
         tbl = Table('testtbl', m,
@@ -1051,6 +1100,40 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "AS string_agg_1 FROM table1"
         )
 
+    def test_pg_array_agg_implicit_pg_array(self):
+
+        expr = pg_array_agg(column('data', Integer))
+        assert isinstance(expr.type, PG_ARRAY)
+        is_(expr.type.item_type._type_affinity, Integer)
+
+    def test_pg_array_agg_uses_base_array(self):
+
+        expr = pg_array_agg(column('data', sqltypes.ARRAY(Integer)))
+        assert isinstance(expr.type, sqltypes.ARRAY)
+        assert not isinstance(expr.type, PG_ARRAY)
+        is_(expr.type.item_type._type_affinity, Integer)
+
+    def test_pg_array_agg_uses_pg_array(self):
+
+        expr = pg_array_agg(column('data', PG_ARRAY(Integer)))
+        assert isinstance(expr.type, PG_ARRAY)
+        is_(expr.type.item_type._type_affinity, Integer)
+
+    def test_pg_array_agg_explicit_base_array(self):
+
+        expr = pg_array_agg(column(
+            'data', sqltypes.ARRAY(Integer)), type_=sqltypes.ARRAY(Integer))
+        assert isinstance(expr.type, sqltypes.ARRAY)
+        assert not isinstance(expr.type, PG_ARRAY)
+        is_(expr.type.item_type._type_affinity, Integer)
+
+    def test_pg_array_agg_explicit_pg_array(self):
+
+        expr = pg_array_agg(column(
+            'data', sqltypes.ARRAY(Integer)), type_=PG_ARRAY(Integer))
+        assert isinstance(expr.type, PG_ARRAY)
+        is_(expr.type.item_type._type_affinity, Integer)
+
     def test_aggregate_order_by_adapt(self):
         m = MetaData()
         table = Table('table1', m, Column('a', Integer), Column('b', Integer))
@@ -1065,6 +1148,22 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "FROM table1 AS foo"
         )
 
+    def test_delete_extra_froms(self):
+        t1 = table('t1', column('c1'))
+        t2 = table('t2', column('c1'))
+        q = delete(t1).where(t1.c.c1 == t2.c.c1)
+        self.assert_compile(
+            q, "DELETE FROM t1 USING t2 WHERE t1.c1 = t2.c1"
+        )
+
+    def test_delete_extra_froms_alias(self):
+        a1 = table('t1', column('c1')).alias('a1')
+        t2 = table('t2', column('c1'))
+        q = delete(a1).where(a1.c.c1 == t2.c.c1)
+        self.assert_compile(
+            q, "DELETE FROM t1 AS a1 USING t2 WHERE a1.c1 = t2.c1"
+        )
+
 
 class InsertOnConflictTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = postgresql.dialect()
@@ -1370,6 +1469,30 @@ class InsertOnConflictTest(fixtures.TestBase, AssertsCompiledSQL):
                                     "param_1": "somename",
                                     "param_2": "unknown"})
 
+    def test_on_conflict_as_cte(self):
+        i = insert(
+            self.table1, values=dict(name='foo'))
+        i = i.on_conflict_do_update(
+            constraint=self.excl_constr_anon,
+            set_=dict(name=i.excluded.name),
+            where=(
+                (self.table1.c.name != i.excluded.name))
+        ).returning(literal_column("1")).cte("i_upsert")
+
+        stmt = select([i])
+
+        self.assert_compile(
+            stmt,
+            "WITH i_upsert AS "
+            "(INSERT INTO mytable (name) VALUES (%(name)s) "
+            "ON CONFLICT (name, description) "
+            "WHERE description != %(description_1)s "
+            "DO UPDATE SET name = excluded.name "
+            "WHERE mytable.name != excluded.name RETURNING 1) "
+            "SELECT i_upsert.1 "
+            "FROM i_upsert"
+        )
+
     def test_quote_raw_string_col(self):
         t = table('t', column("FancyName"), column("other name"))
 
@@ -1448,6 +1571,13 @@ class DistinctOnTest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT DISTINCT ON (t.a) t.id, t.a, t.b FROM t"
         )
 
+    def test_literal_binds(self):
+        self.assert_compile(
+            select([self.table]).distinct(self.table.c.a == 10),
+            "SELECT DISTINCT ON (t.a = 10) t.id, t.a, t.b FROM t",
+            literal_binds=True
+        )
+
     def test_query_plain(self):
         sess = Session()
         self.assert_compile(
