@@ -1,6 +1,6 @@
 
-from sqlalchemy.testing import eq_, assert_raises, \
-    assert_raises_message, is_, is_true
+from sqlalchemy.testing import eq_, le_, assert_raises, \
+    assert_raises_message, is_, is_true, is_false
 from sqlalchemy.ext import declarative as decl
 import sqlalchemy as sa
 from sqlalchemy import testing
@@ -8,10 +8,11 @@ from sqlalchemy import Integer, String, ForeignKey
 from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import relationship, create_session, class_mapper, \
     configure_mappers, clear_mappers, \
-    polymorphic_union, deferred, Session
+    polymorphic_union, deferred, Session, mapper
 from sqlalchemy.ext.declarative import declared_attr, AbstractConcreteBase, \
     ConcreteBase, has_inherited_table
 from sqlalchemy.testing import fixtures, mock
+from test.orm.test_events import _RemoveListeners
 
 Base = None
 
@@ -289,6 +290,95 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
 
         assert class_mapper(Engineer).inherits is class_mapper(Person)
 
+    def test_intermediate_abstract_class_on_classical(self):
+        class Person(object):
+            pass
+
+        person_table = Table('people', Base.metadata,
+                             Column('id', Integer, primary_key=True),
+                             Column('kind', String(50)))
+
+        mapper(Person, person_table,
+               polymorphic_on='kind', polymorphic_identity='person')
+
+        class SpecialPerson(Person):
+            __abstract__ = True
+
+        class Manager(SpecialPerson, Base):
+            __tablename__ = 'managers'
+            id = Column(Integer, ForeignKey(Person.id), primary_key=True)
+            __mapper_args__ = {
+                'polymorphic_identity': 'manager'
+            }
+
+        from sqlalchemy import inspect
+        assert inspect(Manager).inherits is inspect(Person)
+
+        eq_(set(class_mapper(Person).class_manager), {'id', 'kind'})
+        eq_(set(class_mapper(Manager).class_manager), {'id', 'kind'})
+
+    def test_intermediate_unmapped_class_on_classical(self):
+        class Person(object):
+            pass
+
+        person_table = Table('people', Base.metadata,
+                             Column('id', Integer, primary_key=True),
+                             Column('kind', String(50)))
+
+        mapper(Person, person_table,
+               polymorphic_on='kind', polymorphic_identity='person')
+
+        class SpecialPerson(Person):
+            pass
+
+        class Manager(SpecialPerson, Base):
+            __tablename__ = 'managers'
+            id = Column(Integer, ForeignKey(Person.id), primary_key=True)
+            __mapper_args__ = {
+                'polymorphic_identity': 'manager'
+            }
+
+        from sqlalchemy import inspect
+        assert inspect(Manager).inherits is inspect(Person)
+
+        eq_(set(class_mapper(Person).class_manager), {'id', 'kind'})
+        eq_(set(class_mapper(Manager).class_manager), {'id', 'kind'})
+
+    def test_class_w_invalid_multiple_bases(self):
+        class Person(object):
+            pass
+
+        person_table = Table('people', Base.metadata,
+                             Column('id', Integer, primary_key=True),
+                             Column('kind', String(50)))
+
+        mapper(Person, person_table,
+               polymorphic_on='kind', polymorphic_identity='person')
+
+        class DeclPerson(Base):
+            __tablename__ = 'decl_people'
+            id = Column(Integer, primary_key=True)
+            kind = Column(String(50))
+
+        class SpecialPerson(Person):
+            pass
+
+        def go():
+            class Manager(SpecialPerson, DeclPerson):
+                __tablename__ = 'managers'
+                id = Column(Integer,
+                            ForeignKey(DeclPerson.id), primary_key=True)
+                __mapper_args__ = {
+                    'polymorphic_identity': 'manager'
+                }
+
+        assert_raises_message(
+            sa.exc.InvalidRequestError,
+            r"Class .*Manager.* has multiple mapped "
+            r"bases: \[.*Person.*DeclPerson.*\]",
+            go
+        )
+
     def test_with_undefined_foreignkey(self):
 
         class Parent(Base):
@@ -485,6 +575,43 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
                                            ).one(),
             Engineer(name='vlad', primary_language='cobol'))
 
+    def test_single_cols_on_sub_base_of_joined(self):
+        """test [ticket:3895]"""
+
+        class Person(Base):
+            __tablename__ = "person"
+
+            id = Column(Integer, primary_key=True)
+            type = Column(String)
+
+            __mapper_args__ = {
+                "polymorphic_on": type,
+            }
+
+        class Contractor(Person):
+            contractor_field = Column(String)
+
+            __mapper_args__ = {
+                "polymorphic_identity": "contractor",
+            }
+
+        class Employee(Person):
+            __tablename__ = "employee"
+
+            id = Column(Integer, ForeignKey(Person.id), primary_key=True)
+
+        class Engineer(Employee):
+            __mapper_args__ = {
+                "polymorphic_identity": "engineer",
+            }
+
+        configure_mappers()
+
+        is_false(hasattr(Person, 'contractor_field'))
+        is_true(hasattr(Contractor, 'contractor_field'))
+        is_false(hasattr(Employee, 'contractor_field'))
+        is_false(hasattr(Engineer, 'contractor_field'))
+
     def test_single_cols_on_sub_to_joined(self):
         """test [ticket:3797]"""
 
@@ -1011,9 +1138,6 @@ class OverlapColPrecedenceTest(DeclarativeTestBase):
         self._run_test(Engineer, "eid", "pid")
 
 
-from test.orm.test_events import _RemoveListeners
-
-
 class ConcreteInhTest(_RemoveListeners, DeclarativeTestBase):
 
     def _roundtrip(self, Employee, Manager, Engineer, Boss,
@@ -1057,6 +1181,10 @@ class ConcreteInhTest(_RemoveListeners, DeclarativeTestBase):
             eq_(sess.query(Manager).all(), [Manager(name='dogbert')])
             eq_(sess.query(Boss).all(), [Boss(name='pointy haired')])
 
+        e1 = sess.query(Engineer).order_by(Engineer.name).first()
+        sess.expire(e1)
+        eq_(e1.name, 'dilbert')
+
     def test_explicit(self):
         engineers = Table(
             'engineers', Base.metadata,
@@ -1178,6 +1306,55 @@ class ConcreteInhTest(_RemoveListeners, DeclarativeTestBase):
 
         self._roundtrip(Employee, Manager, Engineer, Boss)
 
+    def test_abstract_concrete_extension_descriptor_refresh(self):
+        class Employee(AbstractConcreteBase, Base, fixtures.ComparableEntity):
+            @declared_attr
+            def name(cls):
+                return Column(String(50))
+
+        class Manager(Employee):
+            __tablename__ = 'manager'
+            employee_id = Column(Integer, primary_key=True,
+                                 test_needs_autoincrement=True)
+            paperwork = Column(String(10))
+            __mapper_args__ = {
+                'polymorphic_identity': 'manager', 'concrete': True}
+
+        class Engineer(Employee):
+            __tablename__ = 'engineer'
+            employee_id = Column(Integer, primary_key=True,
+                                 test_needs_autoincrement=True)
+
+            @property
+            def paperwork(self):
+                return "p"
+
+            __mapper_args__ = {
+                'polymorphic_identity': 'engineer', 'concrete': True}
+
+        Base.metadata.create_all()
+        sess = Session()
+        sess.add(Engineer(name='d'))
+        sess.commit()
+
+        # paperwork is excluded because there's a descritor; so it is
+        # not in the Engineers mapped properties at all, though is inside the
+        # class manager.   Maybe it shouldn't be in the class manager either.
+        assert 'paperwork' in Engineer.__mapper__.class_manager
+        assert 'paperwork' not in Engineer.__mapper__.attrs.keys()
+
+        # type currently does get mapped, as a
+        # ConcreteInheritedProperty, which means, "ignore this thing inherited
+        # from the concrete base".   if we didn't specify concrete=True, then
+        # this one gets stuck in the error condition also.
+        assert 'type' in Engineer.__mapper__.class_manager
+        assert 'type' in Engineer.__mapper__.attrs.keys()
+
+        e1 = sess.query(Engineer).first()
+        eq_(e1.name, 'd')
+        sess.expire(e1)
+        eq_(e1.name, 'd')
+
     def test_concrete_extension(self):
         class Employee(ConcreteBase, Base, fixtures.ComparableEntity):
             __tablename__ = 'employee'
