@@ -1,5 +1,5 @@
 from sqlalchemy.testing import eq_, assert_raises, \
-    assert_raises_message, is_
+    assert_raises_message, is_, expect_warnings
 from sqlalchemy.ext import declarative as decl
 import sqlalchemy as sa
 from sqlalchemy import testing
@@ -9,7 +9,8 @@ from sqlalchemy.orm import relationship, create_session, class_mapper, \
     configure_mappers, clear_mappers, \
     deferred, column_property, Session, base as orm_base
 from sqlalchemy.util import classproperty
-from sqlalchemy.ext.declarative import declared_attr
+from sqlalchemy.ext.declarative import declared_attr, declarative_base
+from sqlalchemy.orm import events as orm_events
 from sqlalchemy.testing import fixtures, mock
 from sqlalchemy.testing.util import gc_collect
 
@@ -438,6 +439,90 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
         eq_(MyModel.__table__.kwargs, {'mysql_engine': 'InnoDB'})
 
+    @testing.teardown_events(orm_events.MapperEvents)
+    def test_declare_first_mixin(self):
+        canary = mock.Mock()
+
+        class MyMixin(object):
+            @classmethod
+            def __declare_first__(cls):
+                canary.declare_first__(cls)
+
+            @classmethod
+            def __declare_last__(cls):
+                canary.declare_last__(cls)
+
+        class MyModel(Base, MyMixin):
+            __tablename__ = 'test'
+            id = Column(Integer, primary_key=True)
+
+        configure_mappers()
+
+        eq_(
+            canary.mock_calls,
+            [
+                mock.call.declare_first__(MyModel),
+                mock.call.declare_last__(MyModel),
+            ]
+        )
+
+    @testing.teardown_events(orm_events.MapperEvents)
+    def test_declare_first_base(self):
+        canary = mock.Mock()
+
+        class MyMixin(object):
+            @classmethod
+            def __declare_first__(cls):
+                canary.declare_first__(cls)
+
+            @classmethod
+            def __declare_last__(cls):
+                canary.declare_last__(cls)
+
+        class Base(MyMixin):
+            pass
+        Base = declarative_base(cls=Base)
+
+        class MyModel(Base):
+            __tablename__ = 'test'
+            id = Column(Integer, primary_key=True)
+
+        configure_mappers()
+
+        eq_(
+            canary.mock_calls,
+            [
+                mock.call.declare_first__(MyModel),
+                mock.call.declare_last__(MyModel),
+            ]
+        )
+
+    @testing.teardown_events(orm_events.MapperEvents)
+    def test_declare_first_direct(self):
+        canary = mock.Mock()
+
+        class MyOtherModel(Base):
+            __tablename__ = 'test2'
+            id = Column(Integer, primary_key=True)
+
+            @classmethod
+            def __declare_first__(cls):
+                canary.declare_first__(cls)
+
+            @classmethod
+            def __declare_last__(cls):
+                canary.declare_last__(cls)
+
+        configure_mappers()
+
+        eq_(
+            canary.mock_calls,
+            [
+                mock.call.declare_first__(MyOtherModel),
+                mock.call.declare_last__(MyOtherModel)
+            ]
+        )
+
     def test_mapper_args_declared_attr(self):
 
         class ComputedMapperArgs:
@@ -1103,6 +1188,69 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         TypeA(filters=['foo'])
         TypeB(filters=['foo'])
 
+    def test_arbitrary_attrs_three(self):
+        class Mapped(Base):
+            __tablename__ = 't'
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def some_attr(cls):
+                return cls.__name__ + "SOME ATTR"
+
+        eq_(Mapped.some_attr, "MappedSOME ATTR")
+        eq_(Mapped.__dict__['some_attr'], "MappedSOME ATTR")
+
+    def test_arbitrary_attrs_doesnt_apply_to_abstract_declared_attr(self):
+        names = ["name1", "name2", "name3"]
+
+        class SomeAbstract(Base):
+            __abstract__ = True
+
+            @declared_attr
+            def some_attr(cls):
+                return names.pop(0)
+
+        class M1(SomeAbstract):
+            __tablename__ = 't1'
+            id = Column(Integer, primary_key=True)
+
+        class M2(SomeAbstract):
+            __tablename__ = 't2'
+            id = Column(Integer, primary_key=True)
+
+        eq_(M1.__dict__['some_attr'], 'name1')
+        eq_(M2.__dict__['some_attr'], 'name2')
+
+    def test_arbitrary_attrs_doesnt_apply_to_prepare_nocascade(self):
+        names = ["name1", "name2", "name3"]
+
+        class SomeAbstract(Base):
+            __tablename__ = 't0'
+            __no_table__ = True
+
+            # used by AbstractConcreteBase
+            _sa_decl_prepare_nocascade = True
+
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def some_attr(cls):
+                return names.pop(0)
+
+        class M1(SomeAbstract):
+            __tablename__ = 't1'
+            id = Column(Integer, primary_key=True)
+
+        class M2(SomeAbstract):
+            __tablename__ = 't2'
+            id = Column(Integer, primary_key=True)
+
+        eq_(M1.some_attr, "name2")
+        eq_(M2.some_attr, "name3")
+        eq_(M1.__dict__['some_attr'], 'name2')
+        eq_(M2.__dict__['some_attr'], 'name3')
+        assert isinstance(SomeAbstract.__dict__['some_attr'], declared_attr)
+
 
 class DeclarativeMixinPropertyTest(DeclarativeTestBase):
 
@@ -1177,6 +1325,52 @@ class DeclarativeMixinPropertyTest(DeclarativeTestBase):
         eq_(MyModel.type_.__doc__, """this is a document.""")
         eq_(MyModel.t2.__doc__, """this is another document.""")
 
+    def test_correct_for_proxies(self):
+        from sqlalchemy.ext.hybrid import hybrid_property
+        from sqlalchemy.ext import hybrid
+        from sqlalchemy import inspect
+
+        class Mixin(object):
+            @hybrid_property
+            def hp1(cls):
+                return 42
+
+            @declared_attr
+            def hp2(cls):
+                @hybrid_property
+                def hp2(self):
+                    return 42
+
+                return hp2
+
+        class Base(declarative_base(), Mixin):
+            __tablename__ = 'test'
+            id = Column(String, primary_key=True)
+
+        class Derived(Base):
+            pass
+
+        # in all cases we get a proxy when we use class-bound access
+        # for the hybrid
+        assert Base.hp1._is_internal_proxy
+        assert Base.hp2._is_internal_proxy
+        assert Derived.hp1._is_internal_proxy
+        assert Derived.hp2._is_internal_proxy
+
+        # however when declarative sets it up, it checks for this proxy
+        # and adjusts
+        b1 = inspect(Base)
+        d1 = inspect(Derived)
+        is_(
+            b1.all_orm_descriptors['hp1'],
+            d1.all_orm_descriptors['hp1'],
+        )
+
+        is_(
+            b1.all_orm_descriptors['hp2'],
+            d1.all_orm_descriptors['hp2'],
+        )
+
     def test_column_in_mapper_args(self):
 
         class MyMixin(object):
@@ -1356,7 +1550,7 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
             "SELECT b.x AS b_x, b.x + :x_1 AS anon_1, b.id AS b_id FROM b"
         )
 
-
+    @testing.requires.predictable_gc
     def test_singleton_gc(self):
         counter = mock.Mock()
 
@@ -1392,6 +1586,19 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
             getattr, Mixin, "my_prop"
         )
 
+    def test_can_we_access_the_mixin_straight_special_names(self):
+        class Mixin(object):
+            @declared_attr
+            def __table_args__(cls):
+                return (1, 2, 3)
+
+            @declared_attr
+            def __arbitrary__(cls):
+                return (4, 5, 6)
+
+        eq_(Mixin.__table_args__, (1, 2, 3))
+        eq_(Mixin.__arbitrary__, (4, 5, 6))
+
     def test_non_decl_access(self):
         counter = mock.Mock()
 
@@ -1445,7 +1652,7 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
 
         eq_(counter.mock_calls, [mock.call(A)])
 
-    def test_property_cascade(self):
+    def test_property_cascade_mixin(self):
         counter = mock.Mock()
 
         class Mixin(object):
@@ -1465,6 +1672,67 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
 
         eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
 
+    def test_property_cascade_mixin_override(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr.cascading
+            def my_prop(cls):
+                counter(cls)
+                return column_property(cls.x + 2)
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+
+        with expect_warnings(
+                "Attribute 'my_prop' on class .*B.* "
+                "cannot be processed due to @declared_attr.cascading; "
+                "skipping"):
+            class B(A):
+                my_prop = Column('foobar', Integer)
+
+        eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
+
+    def test_property_cascade_abstract(self):
+        counter = mock.Mock()
+
+        class Abs(Base):
+            __abstract__ = True
+
+            @declared_attr.cascading
+            def my_prop(cls):
+                counter(cls)
+                return column_property(cls.x + 2)
+
+        class A(Abs):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+
+        class B(A):
+            pass
+
+        eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
+
+    def test_warn_cascading_used_w_tablename(self):
+        class Mixin(object):
+            @declared_attr.cascading
+            def __tablename__(cls):
+                return "foo"
+
+        with expect_warnings(
+            "@declared_attr.cascading is not supported on the "
+            "__tablename__ attribute on class .*A."
+        ):
+            class A(Mixin, Base):
+                id = Column(Integer, primary_key=True)
+
+        eq_(A.__table__.name, "foo")
+
     def test_col_prop_attrs_associated_w_class_for_mapper_args(self):
         from sqlalchemy import Column
         import collections
