@@ -8,14 +8,16 @@ from sqlalchemy import testing
 from sqlalchemy.testing import engines
 from sqlalchemy import (
     MetaData, Integer, String, ForeignKey, Boolean, exc, Sequence, func,
-    literal, Unicode, cast)
+    literal, Unicode, cast, DateTime)
 from sqlalchemy.types import TypeDecorator, TypeEngine
 from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.dialects import sqlite
 from sqlalchemy.testing import fixtures
 from sqlalchemy.util import u, b
 from sqlalchemy import util
+from sqlalchemy.testing import mock
 import itertools
+from sqlalchemy.testing.assertsql import CompiledSQL, AllOf, EachOf
 
 t = f = f2 = ts = currenttime = metadata = default_generator = None
 
@@ -329,7 +331,6 @@ class DefaultTest(fixtures.TestBase):
             c = sa.ColumnDefault(fn)
             c.arg("context")
 
-
     @testing.fails_on('firebird', 'Data type unknown')
     def test_standalone(self):
         c = testing.db.engine.contextual_connect()
@@ -427,9 +428,9 @@ class DefaultTest(fixtures.TestBase):
 
         ctexec = sa.select(
             [currenttime.label('now')], bind=testing.db).scalar()
-        l = t.select().order_by(t.c.col1).execute()
+        result = t.select().order_by(t.c.col1).execute()
         today = datetime.date.today()
-        eq_(l.fetchall(), [
+        eq_(result.fetchall(), [
             (x, 'imthedefault', f, ts, ts, ctexec, True, False,
              12, today, 'py', 'hi', 'BINDfoo')
             for x in range(51, 54)])
@@ -447,9 +448,9 @@ class DefaultTest(fixtures.TestBase):
         t.insert().execute({}, {}, {})
 
         ctexec = currenttime.scalar()
-        l = t.select().execute()
+        result = t.select().order_by(t.c.col1).execute()
         today = datetime.date.today()
-        eq_(l.fetchall(),
+        eq_(result.fetchall(),
             [(51, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, 'py', 'hi', 'BINDfoo'),
              (52, 'imthedefault', f, ts, ts, ctexec, True, False,
@@ -463,9 +464,9 @@ class DefaultTest(fixtures.TestBase):
         t.insert().values([{}, {}, {}]).execute()
 
         ctexec = currenttime.scalar()
-        l = t.select().execute()
+        result = t.select().order_by(t.c.col1).execute()
         today = datetime.date.today()
-        eq_(l.fetchall(),
+        eq_(result.fetchall(),
             [(51, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, 'py', 'hi', 'BINDfoo'),
              (52, 'imthedefault', f, ts, ts, ctexec, True, False,
@@ -513,8 +514,8 @@ class DefaultTest(fixtures.TestBase):
 
     def test_insert_values(self):
         t.insert(values={'col3': 50}).execute()
-        l = t.select().execute()
-        eq_(50, l.first()['col3'])
+        result = t.select().execute()
+        eq_(50, result.first()['col3'])
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_updatemany(self):
@@ -533,10 +534,10 @@ class DefaultTest(fixtures.TestBase):
             {'pkval': 52},
             {'pkval': 53})
 
-        l = t.select().execute()
+        result = t.select().execute()
         ctexec = currenttime.scalar()
         today = datetime.date.today()
-        eq_(l.fetchall(),
+        eq_(result.fetchall(),
             [(51, 'im the update', f2, ts, ts, ctexec, False, False,
               13, today, 'py', 'hi', 'BINDfoo'),
              (52, 'im the update', f2, ts, ts, ctexec, True, False,
@@ -550,9 +551,9 @@ class DefaultTest(fixtures.TestBase):
         pk = r.inserted_primary_key[0]
         t.update(t.c.col1 == pk).execute(col4=None, col5=None)
         ctexec = currenttime.scalar()
-        l = t.select(t.c.col1 == pk).execute()
-        l = l.first()
-        eq_(l,
+        result = t.select(t.c.col1 == pk).execute()
+        result = result.first()
+        eq_(result,
             (pk, 'im the update', f2, None, None, ctexec, True, False,
              13, datetime.date.today(), 'py', 'hi', 'BINDfoo'))
         eq_(11, f2)
@@ -562,13 +563,13 @@ class DefaultTest(fixtures.TestBase):
         r = t.insert().execute()
         pk = r.inserted_primary_key[0]
         t.update(t.c.col1 == pk, values={'col3': 55}).execute()
-        l = t.select(t.c.col1 == pk).execute()
-        l = l.first()
-        eq_(55, l['col3'])
+        result = t.select(t.c.col1 == pk).execute()
+        result = result.first()
+        eq_(55, result['col3'])
 
 
 class CTEDefaultTest(fixtures.TablesTest):
-    __requires__ = ('ctes',)
+    __requires__ = ('ctes', 'returning', 'ctes_on_dml')
     __backend__ = True
 
     @classmethod
@@ -635,12 +636,15 @@ class CTEDefaultTest(fixtures.TablesTest):
                 expected
             )
 
+    @testing.requires.ctes_on_dml
     def test_update_in_select(self):
         self._test_a_in_b("update", "select")
 
+    @testing.requires.ctes_on_dml
     def test_delete_in_select(self):
         self._test_a_in_b("update", "select")
 
+    @testing.requires.ctes_on_dml
     def test_insert_in_select(self):
         self._test_a_in_b("update", "select")
 
@@ -671,6 +675,13 @@ class PKDefaultTest(fixtures.TablesTest):
                 default=sa.select([func.max(t2.c.nextid)]).as_scalar()),
             Column('data', String(30)))
 
+        Table(
+            'date_table', metadata,
+            Column(
+                'date_id',
+                DateTime, default=text("current_timestamp"), primary_key=True)
+        )
+
     @testing.requires.returning
     def test_with_implicit_returning(self):
         self._test(True)
@@ -679,20 +690,26 @@ class PKDefaultTest(fixtures.TablesTest):
         self._test(False)
 
     def _test(self, returning):
-        t2, t1 = self.tables.t2, self.tables.t1
+        t2, t1, date_table = (
+            self.tables.t2, self.tables.t1, self.tables.date_table
+        )
 
         if not returning and not testing.db.dialect.implicit_returning:
             engine = testing.db
         else:
             engine = engines.testing_engine(
                 options={'implicit_returning': returning})
-        engine.execute(t2.insert(), nextid=1)
-        r = engine.execute(t1.insert(), data='hi')
-        eq_([1], r.inserted_primary_key)
+        with engine.begin() as conn:
+            conn.execute(t2.insert(), nextid=1)
+            r = conn.execute(t1.insert(), data='hi')
+            eq_([1], r.inserted_primary_key)
 
-        engine.execute(t2.insert(), nextid=2)
-        r = engine.execute(t1.insert(), data='there')
-        eq_([2], r.inserted_primary_key)
+            conn.execute(t2.insert(), nextid=2)
+            r = conn.execute(t1.insert(), data='there')
+            eq_([2], r.inserted_primary_key)
+
+            r = conn.execute(date_table.insert())
+            assert isinstance(r.inserted_primary_key[0], datetime.datetime)
 
 
 class PKIncrementTest(fixtures.TablesTest):
@@ -751,10 +768,10 @@ class PKIncrementTest(fixtures.TablesTest):
         try:
             try:
                 self._test_autoincrement(con)
-            except:
+            except Exception:
                 try:
                     tx.rollback()
-                except:
+                except Exception:
                     pass
                 raise
             else:
@@ -950,6 +967,18 @@ class SequenceDDLTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             "CREATE SEQUENCE foo_seq START WITH 1 MAXVALUE 10 CYCLE",
         )
 
+        self.assert_compile(
+            CreateSequence(Sequence(
+                            'foo_seq', cache=1000, order=True)),
+            "CREATE SEQUENCE foo_seq CACHE 1000 ORDER",
+        )
+
+        self.assert_compile(
+            CreateSequence(Sequence(
+                            'foo_seq', order=True)),
+            "CREATE SEQUENCE foo_seq ORDER",
+        )
+
         self.assert_compile(
             DropSequence(Sequence('foo_seq')),
             "DROP SEQUENCE foo_seq",
@@ -1214,6 +1243,7 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         result = testing.db.execute(t.insert())
         eq_(result.inserted_primary_key, [1])
 
+
 cartitems = sometable = metadata = None
 
 
@@ -1283,6 +1313,85 @@ class TableBoundSequenceTest(fixtures.TestBase):
              (4, "name4", 4)])
 
 
+class SequenceAsServerDefaultTest(
+        testing.AssertsExecutionResults, fixtures.TablesTest):
+    __requires__ = ('sequences_as_server_defaults',)
+    __backend__ = True
+
+    run_create_tables = 'each'
+
+    @classmethod
+    def define_tables(cls, metadata):
+        m = metadata
+
+        s = Sequence("t_seq", metadata=m)
+        Table(
+            "t_seq_test", m,
+            Column("id", Integer, s, server_default=s.next_value()),
+            Column("data", String(50))
+        )
+
+        s2 = Sequence("t_seq_2", metadata=m)
+        Table(
+            "t_seq_test_2", m,
+            Column("id", Integer, server_default=s2.next_value()),
+            Column("data", String(50))
+        )
+
+    def test_default_textual_w_default(self):
+        with testing.db.connect() as conn:
+            conn.execute("insert into t_seq_test (data) values ('some data')")
+
+            eq_(conn.scalar("select id from t_seq_test"), 1)
+
+    def test_default_core_w_default(self):
+        t_seq_test = self.tables.t_seq_test
+        with testing.db.connect() as conn:
+            conn.execute(t_seq_test.insert().values(data="some data"))
+
+            eq_(conn.scalar(select([t_seq_test.c.id])), 1)
+
+    def test_default_textual_server_only(self):
+        with testing.db.connect() as conn:
+            conn.execute(
+                "insert into t_seq_test_2 (data) values ('some data')")
+
+            eq_(conn.scalar("select id from t_seq_test_2"), 1)
+
+    def test_default_core_server_only(self):
+        t_seq_test = self.tables.t_seq_test_2
+        with testing.db.connect() as conn:
+            conn.execute(t_seq_test.insert().values(data="some data"))
+
+            eq_(conn.scalar(select([t_seq_test.c.id])), 1)
+
+    def test_drop_ordering(self):
+        self.assert_sql_execution(
+            testing.db,
+            lambda: self.metadata.drop_all(checkfirst=False),
+            AllOf(
+                CompiledSQL(
+                    "DROP TABLE t_seq_test_2",
+                    {}
+                ),
+                EachOf(
+                    CompiledSQL(
+                        "DROP TABLE t_seq_test",
+                        {}
+                    ),
+                    CompiledSQL(
+                        "DROP SEQUENCE t_seq",   # dropped as part of t_seq_test
+                        {}
+                    ),
+                ),
+            ),
+            CompiledSQL(
+                "DROP SEQUENCE t_seq_2",  # dropped as part of metadata level
+                {}
+            ),
+        )
+
+
 class SpecialTypePKTest(fixtures.TestBase):
 
     """test process_result_value in conjunction with primary key columns.
@@ -1353,9 +1462,15 @@ class SpecialTypePKTest(fixtures.TestBase):
     def test_literal_default_no_label(self):
         self._run_test(default=literal("INT_1", type_=self.MyInteger))
 
+    def test_literal_column_default_no_label(self):
+        self._run_test(default=literal_column("1", type_=self.MyInteger))
+
     def test_sequence(self):
         self._run_test(Sequence('foo_seq'))
 
+    def test_text_clause_default_no_type(self):
+        self._run_test(default=text('1'))
+
     def test_server_default(self):
         self._run_test(server_default='1',)
 
@@ -1596,3 +1711,66 @@ class InsertFromSelectTest(fixtures.TestBase):
             testing.db.execute(table.select().order_by(table.c.x)).fetchall(),
             [(2, 1, 5), (7, 1, 12)]
         )
+
+class CurrentParametersTest(fixtures.TablesTest):
+    __backend__ = True
+
+    @classmethod
+    def define_tables(cls, metadata):
+        def gen_default(context):
+            pass
+
+        Table(
+            "some_table", metadata,
+            Column('x', String(50), default=gen_default),
+            Column('y', String(50)),
+        )
+
+    def _fixture(self, fn):
+
+        def gen_default(context):
+            fn(context)
+        some_table = self.tables.some_table
+        some_table.c.x.default.arg = gen_default
+        return fn
+
+    def _test(self, exec_type, usemethod):
+        collect = mock.Mock()
+
+        @self._fixture
+        def fn(context):
+            collect(context.get_current_parameters())
+
+        table = self.tables.some_table
+        if exec_type in ('multivalues', 'executemany'):
+            parameters = [{"y": "h1"}, {"y": "h2"}]
+        else:
+            parameters = [{"y": "hello"}]
+
+        if exec_type == 'multivalues':
+            stmt, params = table.insert().values(parameters), {}
+        else:
+            stmt, params = table.insert(), parameters
+
+        with testing.db.connect() as conn:
+            conn.execute(stmt, params)
+        eq_(
+            collect.mock_calls,
+            [mock.call({"y": param['y'], "x": None}) for param in parameters]
+        )
+
+    def test_single_w_attribute(self):
+        self._test("single", "attribute")
+
+    def test_single_w_method(self):
+        self._test("single", "method")
+
+    def test_executemany_w_attribute(self):
+        self._test("executemany", "attribute")
+
+    def test_executemany_w_method(self):
+        self._test("executemany", "method")
+
+    @testing.requires.multivalues_inserts
+    def test_multivalued_w_method(self):
+        self._test("multivalues", "method")
